package view;

import java.awt.BorderLayout;
import java.awt.image.BufferedImage;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;

import model.ScaleImageListener;

/**
 * @author Fabrice Bouy
 * @author Mickal Martin Nevot
 * 
 * @version 1.2.0
 */
public class JFrameScaleImageSample1 extends JFrame {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private static final int FRAME_WIDTH = 400;
	private static final int FRAME_HEIGHT = 400;
	
	private static final String BASE_IMG_URI = "img/";  //  @jve:decl-index=0:
	
	private JPanel jContentPane = null;
	
	private JMapPane jMapPane;

	protected BufferedImage image;

	protected JSlider curseur;

	public JFrameScaleImageSample1() {
		super();
		
		initialize();
	}
	
	/**
	 * This method initializes this
	 */
	private void initialize() {
		this.setTitle("MyJFrameScaleImageSample1Title"); // On donne un titre  l'application
		this.setSize(FRAME_WIDTH, FRAME_HEIGHT); // On donne une taille  notre fentre
		this.setLocationRelativeTo(null); // On centre la fentre  l'cran
		this.setResizable(false); // On interdit le redimensionnement de la fentre
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE); // On dit  l'application de se fermer lors du clic sur la croix
		this.setContentPane(getJContentPane());
	}

	/**
	 * This method initializes this.jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (this.jContentPane == null) {
			this.jContentPane = new JPanel();
			this.jContentPane.setLayout(new BorderLayout());
			
			// Initialisation du JPanel
			// Lecture de l'this.image par ImageIO en mode synchrone.
			try {
				this.image = ImageIO.read(getClass().getClassLoader().getResource(BASE_IMG_URI + "java.png"));
			} catch (IOException e1) {
				e1.printStackTrace();
			}
			
			// Ajout du contenu
			this.setJMapPane(new JMapPane(this.image));
			
			// On utilise un JSlider pour l'exemple
			this.curseur = new JSlider(JSlider.VERTICAL, 1, 3, 1);
			this.curseur.setMajorTickSpacing(1);
			this.curseur.setMinorTickSpacing(1);
			this.curseur.setPaintLabels(true);
			this.curseur.setPaintTicks(true);
			this.curseur.setSnapToTicks(true);
			this.curseur.addChangeListener(new ScaleImageListener(this));
			
			// L'utilisation de JScrollPane permet de faire en sorte que l'this.image 
			// ne sorte pas de sa zone lorsqu'un zoom est effectu sur elle
			this.jContentPane.add(new JScrollPane(this.getJMapPane()), BorderLayout.CENTER);
			this.jContentPane.add(this.curseur, BorderLayout.EAST);
		}
		return this.jContentPane;
	}

	public void setJMapPane(JMapPane jMapPane) {
		this.jMapPane = jMapPane;
	}

	public JMapPane getJMapPane() {
		return jMapPane;
	}
}