package view;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;

import javax.swing.JPanel;
import javax.swing.border.Border;

/**
 * Affiche la map.
 *
 * @author Fabrice Bouy
 * @author Mickal Martin Nevot
 * 
 * @version 1.2.0
 */
public class JMapPane extends JPanel {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private BufferedImage image;

	private int scale = 1;

	/**
	 * Cr une nouvelle instance avec l'image passe en paramtre.
	 * 
	 * @param image	l'image initiale.
	 */
	public JMapPane(BufferedImage image) {
		super();
		setImage(image);
	}

	/**
	 * Recalculer correctement la taille du panel.
	 */
	private void recalculateAndResetSize() {
		int width = 0;
		int height = 0;
		
		// Tenir compte de la bordure optionnelle du panel.
		Insets insets = getInsets();
		width += insets.left + insets.right;
		height += insets.top + insets.bottom;
		
		// Tenir compte de la taille de l'image.
		if (this.image != null) {
			width += this.image.getWidth() * this.scale;
			height += this.image.getHeight() * this.scale;
		}
		
//		Mon_image_zoome = image_initial.getScaledInstance(width, height, Image.SCALE_SMOOTH); 
		
		Dimension size = new Dimension(width, height);
		setSize(size);
		setMinimumSize(size);
		setPreferredSize(size);
	}

	// Getters/Setters.

	/**
	 * Modifie l'image de fond
	 * 
	 * @param image la nouvelle image.
	 * @see #getImage
	 */
	public void setImage(BufferedImage image) {
		this.image = image;
		recalculateAndResetSize();
		repaint();
	}

	/**
	 * Renvoie l'image de fond
	 * 
	 * @return une instance de <code>BufferedImage</code>.
	 * @see #setImage
	 */
	public BufferedImage getImage() {
		return this.image;
	}

	/**
	 * Modifie le facteur de zoom  la valeur passe en paramtre.
	 * 
	 * @param value	la nouvelle valeur de zoom.
	 * @see #getScale
	 */
	public void setScale(int value) {
		this.scale = value;
		recalculateAndResetSize();
		repaint();
	}

	/**
	 * Renvoie le facteur de zoom
	 * 
	 * @return un entier.
	 * @see #setScale
	 */
	public int getScale() {
		return this.scale;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setBorder(Border border) {
		super.setBorder(border);
		recalculateAndResetSize();
	}

	// Rendu graphique.

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void paintComponent(Graphics graphics) {
		super.paintComponent(graphics);
		
		Graphics2D g2d = (Graphics2D) graphics;
		
		if (this.image != null) {
			Insets insets = getInsets();
			Dimension size = getSize();
			int panelWidth = size.width - (insets.left + insets.right);
			int panelHeight = size.height - (insets.top + insets.bottom);
			int scaledImageWidth = this.image.getWidth() * this.scale;
			int scaledImageHeight = this.image.getHeight() * this.scale;
			int x = (panelWidth - scaledImageWidth) / 2;
			int y = (panelHeight - scaledImageHeight) / 2;
			
			// Pour permettre au bord optionnel d'etre dessine.
			g2d.translate(insets.left, insets.top);
			g2d.setRenderingHint(RenderingHints.KEY_RENDERING,
					RenderingHints.VALUE_RENDER_QUALITY);
			g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
					RenderingHints.VALUE_ANTIALIAS_OFF);
			g2d.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
					RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);
			g2d.drawImage(this.image, x, y, scaledImageWidth, scaledImageHeight, null);
			
			// Retour  l'origine du repre.
			g2d.translate(-insets.left, -insets.top);
		}
	}
}